package solutions;

import java.net.URI;
import java.util.concurrent.CompletableFuture;

import jdk.incubator.http.HttpClient;
import jdk.incubator.http.HttpRequest;
import jdk.incubator.http.HttpResponse;
import jdk.incubator.http.HttpResponse.BodyHandler;

/**
 * Übung für JUGS-Java9-Workshop
 * 
 * @author Michael Inden
 * 
 *         Copyright 2017 by Michael Inden
 */
public class Exercise10_Http2_Async_Bonus
{
    public static void main(final String[] args) throws Exception
    {
        final URI uri = new URI("https://www.oracle.com/index.html");

        final HttpRequest request = HttpRequest.newBuilder(uri).GET().build();
        final BodyHandler<String> asString = HttpResponse.BodyHandler.asString();

        final HttpClient httpClient = HttpClient.newHttpClient();
        final CompletableFuture<HttpResponse<String>> asyncResponse = 
                                                      httpClient.sendAsync(request, 
                                                                           asString);
        asyncResponse.thenAccept(response -> printResponseInfo(response));

        waitForCompletion();
        if (asyncResponse.isDone())
        {
            final HttpResponse<String> response = asyncResponse.get();
            printResponseInfo(response);
        }
        else
        {
            asyncResponse.cancel(true);
            System.err.println("timeout");
        }
    }

    private static void waitForCompletion() throws InterruptedException
    {
        for (int i = 0; i < 10; i++)
        {
            System.out.println("Step " + i);
            Thread.sleep(200);
        }
    }

    private static void printResponseInfo(final HttpResponse<String> response)
    {
        final int responseCode = response.statusCode();
        final String responseBody = response.body();

        System.out.println("Status: " + responseCode);
        System.out.println("Body:   " + responseBody);
    }
}
